package Package::Autoloader::Generator::Hello_Anything;
use strict;
use warnings;
use parent qw(
	Package::Autoloader::Generator
);

sub new {
	my ($class, $defining_pkg) = (shift, shift);

        my $generator = sub {
		my ($pkg, $sub_name) = (shift, shift);

                my $sub_body = sprintf(
			qq{print 'Hello %s\n'},
			substr($sub_name, 6) || '');
 		return($sub_body);
        };
	my $self = [$generator];
	bless($self, $class);
	Internals::SvREADONLY(@{$self}, 1);

	return($self);
}

1;